import os
import sys
from typing import  Literal

import time
import logging

# 将项目根目录（pyinsts）添加到Python路径
sys.path.append(os.path.abspath(os.path.join(os.path.dirname(__file__), '../../../../')))
from src.pyinsts.libs.baseinstrument import BaseInstrument




class N9020b(BaseInstrument):
    def __init__(self, address: str = None, config_path="config.yaml", model: str = "N9030B"):
        super().__init__(address=address, config_path=config_path, model=model)

        self.model = model
    def set_mode_spectrum_analyzer(self, mode:Literal["SAN"]):
        """
        设置频谱模式
        :param mode:
        :return:
        """
        self.write(f'CONF:{mode}; *WAI')  # 设置后使用 *WAI 等待命令执行完成 {mode}')

    def set_freq(self, freq:float, unit:Literal['GHz', 'MHz', 'kHz', 'Hz']):
        """
        设置频率
        :param unit:
        :param freq:
        :return:
        """

        self.write(f'SENS:FREQ {freq}{unit}')
        logging.info(f'设置频率: {freq}{unit}')

    def set_freq_cent(self, freq:float, unit:Literal['GHz', 'MHz', 'kHz', 'Hz']):
        """
        设置中心频率
        :param unit:
        :param freq:
        :return:
        """

        self.write(f'FREQ:CENT {freq}{unit}',True)
        logging.info(f'设置中心频率: {freq}{unit}')


    def set_freq_span(self, freq:float , unit:Literal['GHz', 'MHz', 'kHz', 'Hz']):
        """
        设置频谱的跨度 (span)，单位为 GHz
        :param unit:
        :param freq:
        :return:
        """

        self.write(f'FREQ:SPAN {freq}{unit}; *WAI')  # 设置后使用 *WAI 等待命令执行完成

        logging.info(f'已设置频谱跨度为: {freq}{unit}')


    def set_freq_span_max(self):
        """
        设置频谱的跨度 (span)为最大
        :return:
        """
        self.write(f'FREQ:SPAN MAX; *WAI')  # 设置后使用 *WAI 等待命令执行完成
        time.sleep(0.001)  # 稍作延迟以确保指令被正确执行
        logging.info(f'已设置频谱跨度为max')

    def set_freq_start(self, freq: float, unit:Literal['GHz', 'MHz', 'kHz', 'Hz']):
        """
        设置开始频率
        :param unit:
        :param freq:
        :return:
        """

        self.write(f'FREQ:STAR {freq}{unit}')
        logging.info(f'设置起始频率: {freq}{unit}')


    def set_freq_stop(self, freq: float, unit:Literal['GHz', 'MHz', 'kHz', 'Hz']):
        """
        设置截止频率
        :param unit:
        :param freq:
        :return:
        """

        self.write(f'FREQ:STOP {freq}{unit}')
        logging.info(f'设置结束频率: {freq}{unit}')


    def set_ref_level(self, level: float):
        """
        设置参考电平 (Reference Level)
        :param level: 参考电平值，单位 dBm
        :return:
        """

        self.write(f'DISP:WIND:TRAC:Y:RLEV {level}dBm')  # 设置参考电平
        logging.info(f'已设置参考电平为: {level} dBm')


    def set_auto_tune(self):
        """
        自动设置参考电平
        :return:
        """

        self.write('SENS:FREQ:TUNE:IMM; *WAI')  # 设置后使用 *WAI 等待命令执行完成
        logging.info('已设置自动调频')

    def set_rbw(self, rbw: float, unit:Literal['MHz', 'kHz', 'Hz']):
        """
        设置分辨带宽 (Resolution Bandwidth, RBW)
        :param rbw: 分辨带宽值
        :param unit:
        :return:
        """
        self.write(f'BAND:RES {rbw}{unit}; *WAI')  # 设置后使用 *WAI 等待命令执行完成
        logging.info(f'已设置分辨带宽为: {rbw}{unit}')


    def set_rbw_auto(self):
        """
        设置自动rbw
        :return:
        """
        try:
            self.write('BWID:AUTO ON')  # 设置 RBW 为自动模式并等待
            logging.info('已将分辨率带宽设置为自动模式')
        except Exception as e:
            logging.error(f"设置 RBW 时发生错误: {e}")


    def set_vbw(self, vbw: float, unit:Literal['MHz', 'kHz', 'Hz']):
        """
        设置视频带宽 (Video Bandwidth, VBW)
        :param vbw: 视频带宽值
        :param unit: 带宽的单位，默认是 Hz（可选 kHz, MHz）
        :return:
        """

        try:
            # 构建 SCPI 命令
            self.write(f'BAND:VID {vbw}{unit}; *WAI')  # 设置后使用 *WAI 等待命令执行完成
            logging.info(f'已设置视频带宽为: {vbw} {unit}')
        except Exception as e:
            logging.error(f"设置视频带宽时发生错误: {e}")

    def set_vbw_auto(self):
        """
        自动设置vbw
        :return:
        """

        try:
            self.write('BAND:VID AUTO; *WAI')  # 设置 VBW 为自动模式并等待
            logging.info('已将视频带宽设置为自动模式')
        except Exception as e:
            logging.error(f"设置 VBW 时发生错误: {e}")


    def set_peak_search(self):
        """
        peak search
        :return:
        """
        self.instrument.write(f"CALC:MARK1:MAX:PEAK")
        logging.info('执行peak search')
    def set_next_peak(self):
        """
        next peak
        :return:
        """
        self.write(f"CALC:MARK1:MAX:NEXT")
        logging.info('执行next peak')

    def set_marker_freq(self, frequency: float,unit: Literal['GHz', 'MHz', 'kHz', 'Hz'] = 'Hz', mark_number: int = 1):
        """
        设置指定mark标记的频率
        :param unit:
        :param frequency: 频率值
        :param mark_number: 标记编号，默认为 1
        :return:
        """
        try:
            self.write(f'CALC:MARK{mark_number}:FREQ {frequency}{unit}; *WAI')  # 设置指定标记的频率并等待
            logging.info(f'已将标记{mark_number}的频率设置为: {frequency}{unit}')
        except Exception as e:
            logging.error(f"设置标记{mark_number}频率时发生错误: {e}")

    def set_mark_delt_on(self, x):
        """
        开启delt标记
        :param x:
        :return:
        """
        self.write(f"CALC:MARK{x}:MODE DELT")
        logging.info(f'开启delt标记: MARK{x}')

    def set_mark_all_off(self):
        """
        关闭所有mark
        :return:
        """
        self.write(f"CALC:MARK:AOFF")
        logging.info('关闭所有标记')


    def set_mark_del_freq(self, mark_number: int, freq:float, unit: Literal['GHz', 'MHz', 'kHz', 'Hz']):
        """
        delt频率
        :param unit:
        :param mark_number:1,2,3
        :param freq:
        :return:
        """

        self.write(f"CALC:MARK{mark_number}:X {freq}{unit}")
        logging.info(f'设置MARK{mark_number}的delt频率为: {freq}{unit}')


    def query_mark_delta_x(self, mark_number: int, unit: Literal['GHz', 'MHz', 'kHz', 'Hz']):
        """
        读取mark delt
        :param mark_number:
        :param unit:
        :return:
        """
        value = self.query(f'CALC:MARK{mark_number}:X?')
        if unit == 'GHz':
            value = float(value) / 1e9
        elif unit == 'MHz':
            value = float(value) / 1e6
        elif unit == 'kHz':
            value = float(value) / 1e3
        elif unit == 'Hz':
            value = float(value)
        logging.info(f'读取MARK{mark_number}的偏移频率: {value}{unit}')
        return value, unit


    def query_mark_x_freq(self, mark_number: int, unit: Literal['GHz', 'MHz', 'kHz', 'Hz']):
        """
        读取mark频率
        :param mark_number:
        :param unit:
        :return: 返回频率值和单位
        """

        value = self.query(f"CALC:MARK{mark_number}:X?")
        if unit == 'GHz':
            value = float(value.replace("\n", "").replace("\r", "")) / 1e9  # 删除字符串中的换行符和回车符
        elif unit == 'MHz':
            value = float(value.replace("\n", "").replace("\r", "")) / 1e6
        elif unit == 'kHz':
            value = float(value.replace("\n", "").replace("\r", "")) / 1e3
        elif unit == 'Hz':
            value = float(value.replace("\n", "").replace("\r", ""))
        logging.info(f'读取MARK频率为:{value}{unit}')
        return value, unit


    def query_mark_y_power(self):
        """
        读取mark功率,单位dBm
        :return:
        """
        value = self.query(f"CALC:MARK:Y?")
        value = float(value.replace("\n", "").replace("\r", ""))  # 删除字符串中的换行符和回车符
        logging.info(f'读取MARK频率为:{value} dBm')
        return value

    def set_peak_search_continuous_on(self):
        """
        设置peak search连续最终打开
        :return:
        """
        self.write(f"CALC:MARK:CPS ON")
        logging.info(f'set_peak_search_continuous 成功')

    def set_mark_to_cf(self,mark_number: int = 1):
        """
        将指定mark设置到中心频率
        :param mark_number:
        :return:
        """
        self.write(f"CALC:MARK{mark_number}:CENT")
        logging.info(f'CALC:MARK{mark_number}:CENT 成功')


    def save_png(self, file_path):
        """
        保存数据图片和csv文件
        :param file_path:
        :return:
        """
        self.write(f'MMEM:STOR:TRAC:DATA TRACE1, "{file_path}.csv"; *WAI')
        self.write(f'MMEM:STOR:SCR "{file_path}.PNG"; *WAI')

        logging.info(f'保存屏幕截图到: {file_path}.PNG')

        time.sleep(1)

    def set_single(self):
        """
        单次运行
        :return:
        """
        self.write(f"INIT:CONT 0")
        logging.info(f'set_single 成功')


    def set_cont(self):
        """
        连续运行
        :return:
        """
        self.write(f"INIT:CONT 1")
        logging.info(f'set_cont 成功')

    def set_cal_all(self):
        """
        自校准all
        :return:
        """
        self.write(f"*CAL?",True)
        logging.info(f'自校准all 成功')

    def close(self):
        """
        关闭仪器端口
        :return:
        """
        self.close()
        logging.info(f'关闭频谱仪{self.model}端口')


if __name__ == '__main__':
    n9020b = N9020b(address="USB0::0x2A8D::0x1D0B::MY55480186::INSTR", model='N9020B')
    n9020b.set_rbw_auto()
    n9020b.set_freq_cent(1,'GHz')   # 设置中心频率
    # n9020b.set_cont()














"""
日志处理
"""
import logging
import os
from logging.handlers import RotatingFileHandler
import colorlog  # 导入 colorlog
def setup_logging(log_file=None, log_level=logging.INFO, max_bytes=1 * 1024 * 1024, backup_count=5):
    """
    配置日志记录，包括日志轮换和控制台输出，支持彩色日志
    :param log_file: 日志文件路径，如果为None则使用默认路径
    :param log_level: 日志等级
    :param max_bytes: 单个日志文件的最大字节数，默认为1MB
    :param backup_count: 保留的日志文件备份数
    """
    if log_file is None:
        log_file = os.getenv('LOG_FILE_PATH', 'docs/log/test.log')

    # 确保日志文件夹存在
    log_dir = os.path.dirname(log_file)
    if not os.path.exists(log_dir):
        os.makedirs(log_dir)

    # 创建根记录器
    logger = logging.getLogger()
    logger.setLevel(log_level)

    # 清除已有的处理器，避免重复添加
    if logger.hasHandlers():
        logger.handlers.clear()

    # 创建文件处理器，支持日志轮换
    file_handler = RotatingFileHandler(log_file, mode='w', maxBytes=max_bytes, backupCount=backup_count)
    file_handler.setLevel(log_level)

    # 创建控制台处理器，日志输出到控制台
    console_handler = logging.StreamHandler()
    console_handler.setLevel(log_level)

    # 定义日志格式
    log_format = '%(asctime)s - %(levelname)-8s - %(message)-60s - [%(filename)-15s:%(lineno)-4d]'
    color_format = "%(log_color)s" + log_format

    # 设置文件处理器的格式
    file_formatter = logging.Formatter(log_format)
    file_handler.setFormatter(file_formatter)

    # 设置控制台处理器的彩色格式
    color_formatter = colorlog.ColoredFormatter(
        color_format,
        log_colors={
            'DEBUG': 'cyan',
            'INFO': 'green',
            'WARNING': 'yellow',
            'ERROR': 'red',
            'CRITICAL': 'bold_red',
        }
    )
    console_handler.setFormatter(color_formatter)

    # 将处理器添加到记录器中
    logger.addHandler(file_handler)
    logger.addHandler(console_handler)

    # 添加日志分隔符
    logger.info("=" * 60)
    logger.info("新日志开始".center(60, ' '))
    logger.info("=" * 60)


def log_data(data):
    """
    记录数据的日志
    """
    logging.info(f"数据记录: {data}")


# 通用异常处理装饰器
def log_exceptions(func):
    def wrapper(*args, **kwargs):
        try:
            return func(*args, **kwargs)
        except Exception as e:
            logging.error(f"{func.__name__} 执行时发生错误", exc_info=True)
            raise e

    return wrapper

# 示例：调用日志设置
if __name__ == "__main__":
    setup_logging(log_file='/Users/zhangxianping/Desktop/pythonProject/SiCore_ChipTest/docs/log//test.log', log_level=logging.DEBUG)
    log_data("这是一个测试数据记录")
    logging.debug("这是一条调试信息")
    logging.warning("这是一条警告信息")
    logging.error("这是一条错误信息")
    logging.critical("这是一条严重错误信息")


"""
@date: 2024-11-24
@author: zhangxp93
@contact: 937097345@qq.com

您可以免费地使用、修改和分发PyInst ，但必须包含原始作者的版权声明和下列许可声明。

You may use, modify, and distribute pyinsts free of charge, but you must include
the original author's copyright notice and the following license notice.
"""

"""
MIT License

Copyright (c) [2024] [zhangxp93]

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
"""

import logging
import pyvisa
import time



# 定义Fswp类，连接Fswp信号源分析仪
class Fswp:
    def __init__(self, instrument_address):
        """
        设置Fswp仪器地址，连接Fswp信号源分析仪
        :param instrument_address:写入仪器地址，例如："USB0::0x0957::0x0E09::MY47402409::0::INSTR"
        """
        self.instrument_address = instrument_address
        self.rm = pyvisa.ResourceManager()
        self.Fswp = self.rm.open_resource(self.instrument_address)
        time.sleep(0.01)
        response = self.Fswp.query("*IDN?")
        response = response.replace("\n", "").replace("\r", "")  # 删除字符串中的换行符和回车符
        logging.info(f'Instrument identification:{response}')
        logging.info('FSWP连接成功')

    def run_cont(self):
        """
        cont运行
        :return:
        """
        self.Fswp.write('INIT:CONT OFF')
        self.Fswp.write('INIT:CONT ON')

    def run_single(self):
        """
        RUN SINGLE
        :return:
        """
        self.Fswp.write('INIT:CONT OFF')
        self.Fswp.write('INIT:IMM')

    def set_run_single1(self):
        """
        不使用了
        RUN SINGLE
        :return:
        """
        self.Fswp.write('INIT:CONT OFF')
        self.Fswp.write('INIT:IMM')
        #
        star_time = time.time()
        while True:
            opc = self.Fswp.query("*OPC?")
            if opc.strip() == '1':
                logging.info('run single运行完成')
                break
            if time.time() - star_time > 60:
                logging.error(f'运行超时')
            time.sleep(1)

    def set_run_single(self):
        """
        设置仪器为单次运行模式，并等待运行完成，同时记录运行时间。
        """
        try:
            # 设置 PyVISA 的超时时间为 10 秒
            self.Fswp.timeout = 10000  # 单位为毫秒，10 秒

            # 记录开始时间
            start_time = time.time()

            # 发送 SCPI 指令：关闭连续运行，启动单次运行
            self.Fswp.write('INIT:CONT OFF')
            self.Fswp.write('INIT:IMM')

            # 轮询 OPC 状态
            while True:
                opc = self.Fswp.query("*OPC?")
                if opc.strip() == '1':  # 操作完成
                    logging.info('run single 运行完成')
                    break

                # 检查是否超时（超时时间为 60 秒）
                if time.time() - start_time > 60:
                    logging.error('运行超时')
                    break

                # 每 1 秒检查一次
                time.sleep(0.5)

            # 记录结束时间并计算运行时间
            end_time = time.time()
            elapsed_time = end_time - start_time  # 运行时间，单位为秒
            logging.info(f'运行时间: {elapsed_time:.2f} 秒')

        except pyvisa.errors.VisaIOError as e:
            logging.error(f"VisaIOError: {e}")
        except Exception as e:
            logging.error(f"运行出错: {e}")

    def set_cent_freq(self, freq):
        """
        设置中心频率,单位GHz,PN和SP模式都可用
        :param freq:
        :return:
        """
        self.Fswp.write(f'FREQ:CENT {freq}GHz')
        time.sleep(0.001)

    def set_window(self, window):
        """
        选择窗口
        :param window:
        :return:
        """
        self.Fswp.write(f'INST "{window}"')

    def set_marker_x(self, x, offset):
        """
        设置marker
        :param x:
        1,2,3,4,5,6,7
        :param offset:1K,10K,100K,1M,10M,20M,100M
        :return:
        """
        marker_on = self.Fswp.write(f'CALC:MARK{x} ON')  # 打开marker
        self.Fswp.write(f'DISP:MTAB ON')  # 显示marker表
        self.Fswp.write(f'CALC:MARK{x}:X {offset}Hz')
        time.sleep(0.01)

    def query_marker_x(self, x):
        """
        读取mark值
        :param x: 1,2,3,4,5,6,7
        :return:
        """
        marker_value = self.Fswp.query(f'CALC:MARK{x}:Y?')
        return marker_value

    def query_rms(self):
        """
        读取积分抖动rms,fs
        :return:
        """
        jitter_value = float(self.Fswp.query('FETC:RANG2:PNO2:RMS?')) * 1e15
        print('jitter时间：', jitter_value)
        return jitter_value

    def save_csv_png(self, filename):
        """
        存csv与png
        :param filename: 文件名需包含路径，'Z:\\DATA\\186_5514\\186_5514_SIOA201P8_GND_REF300_12000_+35'
        :return:
        """
        self.Fswp.write(f'MMEM:STOR1:TRAC 1,"{filename}.csv"')
        self.Fswp.write(f'MMEM:NAME "{filename}.png"')
        self.Fswp.write('HCOP:IMM')
        logging.info('save_csv_png 运行完成')

    def save_png(self, filename):
        """
        存csv与png
        :param filename: 文件名需包含路径，'Z:\\DATA\\186_5514\\186_5514_SIOA201P8_GND_REF300_12000_+35'
        :return:
        """
        self.Fswp.write(f'MMEM:NAME "{filename}.png"')
        self.Fswp.write('HCOP:IMM')
        logging.info('save_png 运行完成')

    def set_dc_power(self, on_off):
        """
        设置DC POWER
        :param on_off:ON,OFF
        :return:
        """

        self.Fswp.write(f'SOUR:VOLT {on_off}')

    def set_dc_supply_volt(self, volt):
        """
        设置DC voltage supply
        :param volt:电压值
        :return:
        """

        self.Fswp.write(f'SOUR:VOLT:POW:LEV:AMPL {volt}')

    def set_dc_power_on(self):
        """
        设置DC POWER ON
        :return:
        """
        self.Fswp.write(f'SOUR:VOLT ON')

    def set_dc_power_off(self):
        """
        设置DC POWER off
        :return:
        """
        self.Fswp.write(f'SOUR:VOLT OFF')

    def set_vtune(self, vt):
        """
        设置vt电压
        :param vt:
        :return:
        """
        self.Fswp.write(f'SOUR:VOLT:CONT:LEV:AMPL {vt}')

    def set_auto_search_off(self):
        """
        auto_search关闭
        :return:
        """

        self.Fswp.write(f'SENS:ADJ:CONF:FREQ:AUT:STAT OFF')

    def query_freq_cent(self):
        """
        查询频率
        :return:
        """
        freq = float(self.Fswp.query(f'FREQ:CENT?')) / 1e9
        return freq

    def query_singal_power(self):
        power = float(self.Fswp.query(f'POW:RLEV?'))
        return power

    def close(self):
        self.Fswp.close()
        # self.rm.close()
        # print('关闭Fswp连接')
        logging.info('关闭Fswp连接')

    def set_select_pn(self):
        """
        选择相噪模式窗口
        :return:
        """
        self.Fswp.write('INST PNO')

    def set_select_sp(self):
        """
        选择频谱模式窗口
        :return:
        """
        self.Fswp.write('INST SAN')

    def set_peak_search(self):
        self.Fswp.write('CALC:MARK:MAX')

    def set_mark_delt(self):
        self.Fswp.write('CALC:DELT ON')

    def set_mark_delt_freq(self, x, freq):
        self.Fswp.write(f'CALC:DELT{x}:X {freq}MHz')

    def query_mark_delt_x(self, x):
        """
        读取偏移频率
        :return:
        """
        value = self.Fswp.query(f'CALC:DELT{x}:X:REL?')
        return value

    def query_mark_delt_y(self, y):
        value = float(self.Fswp.query(f'CALC:DELT{y}:Y?'))
        return value

    def set_continuous_peak(self):
        self.Fswp.write('CALC:MARK:MAX:AUTO ON')

    def set_mark_all_off(self):
        """
        关闭所有mark
        :return:
        """
        self.Fswp.write('CALC:MARK:AOFF')

    def set_freq_span(self, span):
        """
        sp模式设置span,MHz
        :return:
        """
        self.Fswp.write(f'SENS:FREQ:SPAN {span}MHz')

    def set_bw(self, bw):
        self.Fswp.write(f'SENS:BAND:RES {bw}Hz')

    def set_mark_on(self, x):
        self.Fswp.write(f'CALC:MARK{x}:STAT ON')
        time.sleep(0.01)

    def set_mark_del_on(self, x):
        self.Fswp.write(f'CALC:DELT{x}:STAT ON')
        time.sleep(0.01)

    def set_disp_rlev(self, value):
        self.Fswp.write(f'DISP:WIND:TRAC:Y:SCAL:RLEV {value}')

    def set_signal_source_freq(self, freq):
        """
        设置源频率，GHz
        :param freq:
        :return:
        """
        self.Fswp.write(f'SOUR:GEN:FREQ {freq}MHz')

    def set_signal_source_pow(self, power):
        """
        设置源功率，dBm
        :return:
        """
        self.Fswp.write(f'SOUR:GEN:LEV {power}')

    def set_signal_source_on(self, on_off):
        """
        设置源功率开关,ON,OFF
        :return:
        """
        self.Fswp.write(f'SOUR:GEN:STAT {on_off}')

    def query_vcochar_freq(self):
        """
        读取trac1,freq
        :return:
        """
        freq_value = self.Fswp.query("TRAC1? TRACE1")
        freq_value = freq_value.split(',')
        freq_value = [freq_value[i] for i in range(1, len(freq_value), 2)]  # 第一个值为vt电压，第二个值为freq，步进为2
        freq_value = [float(item) / 1e9 for item in freq_value]
        return freq_value

    def query_vcochar_power(self):
        """
        读取trac2,功率
        :return:
        """
        power = self.Fswp.query("TRAC2? TRACE1")
        power = power.split(',')
        power = [power[i] for i in range(1, len(power), 2)]
        return power

    def query_vcochar_sen(self):
        """
        读取trac3,调谐灵敏度
        :return:
        """
        sen = self.Fswp.query(f"TRAC3? TRACE1")
        sen = sen.split(',')
        sen = [sen[i] for i in range(1, len(sen), 2)]
        sen = [float(item) / 1e6 for item in sen]
        return sen

    def query_vcochar_vt(self):
        """
        读取trac1
        :return:
        """
        vt = self.Fswp.query("TRAC1? TRACE1")
        vt = vt.split(',')
        vt = [vt[i] for i in range(0, len(vt), 2)]
        return vt

    def query_vcochar_icc(self):
        """
        读取trac4
        :return:
        """
        icc = self.Fswp.query(f"TRAC4? TRACE1")
        icc = icc.split(',')
        icc = [icc[i] for i in range(1, len(icc), 2)]
        icc = [float(item) / 1e6 for item in icc]
        return icc


class FswpVcoChar(Fswp):
    def __init__(self, instrument_address):
        super().__init__(instrument_address)

    def query_freq(self):
        """
        读取trac1
        :return:
        """
        freq = self.Fswp.query("TRAC1? TRACE1")
        freq = freq.split(',')
        freq = [freq[i] for i in range(1, len(freq), 2)]
        freq = [float(item) / 1e9 for item in freq]
        return freq

    def query_vt(self):
        """
        读取trac1
        :return:
        """
        vt = self.Fswp.query("TRAC1? TRACE1")
        vt = vt.split(',')
        vt = [vt[i] for i in range(0, len(vt), 2)]
        return vt

    def query_power(self):
        """
        读取trac2
        :return:
        """
        power = self.Fswp.query("TRAC2? TRACE1")
        power = power.split(',')
        power = [power[i] for i in range(1, len(power), 2)]
        return power

    def query_sen(self):
        """
        读取trac3,调谐灵敏度
        :return:
        """
        sen = self.Fswp.query(f"TRAC3? TRACE1")
        sen = sen.split(',')
        sen = [sen[i] for i in range(1, len(sen), 2)]
        sen = [float(item) / 1e6 for item in sen]
        return sen

    def query_icc(self):
        """
        读取trac4
        :return:
        """
        icc = self.Fswp.query(f"TRAC4? TRACE1")
        icc = icc.split(',')
        icc = [icc[i] for i in range(1, len(icc), 2)]
        icc = [float(item) / 1e6 for item in icc]
        return icc

    def set_freq_search_range(self, freq_low, freq_high):
        self.Fswp.write(f"ADJ:CONF:FREQ:LIM:LOW {freq_low}GHz")
        self.Fswp.write(f"ADJ:CONF:FREQ:LIM:HIGH {freq_high}GHz")


class SpotNoiseTune(Fswp):
    def __init__(self, instrument_address):
        super().__init__(instrument_address)

    def query_phase(self, trace):
        phase = self.Fswp.query(f"TRAC1? TRACE{trace}")
        phase = phase.split(',')
        phase = [phase[i] for i in range(1, len(phase), 2)]
        phase[-1] = phase[-1].rstrip("\n")
        return phase

    def query_phase_vt(self, trace):
        phase_vt = self.Fswp.query(f"TRAC1? TRACE{trace}")
        phase_vt = phase_vt.split(',')
        phase_vt = [phase_vt[i] for i in range(0, len(phase_vt), 2)]
        return phase_vt


if __name__ == '__main__':
    addr_fswp = 'TCPIP0::172.16.30.150::inst0::INSTR'
    fswp = Fswp(addr_fswp)
    fswp.set_cent_freq(10)
    fswp.close()



import logging
import time

import pyvisa
from typing import Callable, Optional
import os
import inspect
from functools import wraps

from pyvisa import VisaIOError

from src.pyinsts.libs.data import load_config


def handle_instrument_error(func: Callable) -> Callable:
    """仪器操作错误处理装饰器"""
    @wraps(func)
    def wrapper(*args, **kwargs):
        try:
            return func(*args, **kwargs)
        except Exception as e:
            logging.error(f"{args[0].model} {func.__name__}失败: {e}")
            raise

    return wrapper


class BaseInstrument:
    """基础仪器控制类"""

    def __init__(self, address: Optional[str] = None, config_path: str = "config.yaml",
                 model: str = "E5052B"):
        """初始化仪器连接

        Args:
            address: 仪器地址，例如："USB0::0x0957::0x0E09::MY47402409::0::INSTR"
            config_path: 配置文件路径
            model: 仪器型号
        """
        # 设置基本属性
        self.model = model
        self.address = self._get_instrument_address(address, config_path)
        
        # 设置默认超时参数

        self.opc_timeout = 100  # 默认OPC超时时间100秒


        self.opc_poll_interval = 0.01  # 默认OPC轮询间隔0.01秒

        # 连接仪器
        self.idn = self._connect_instrument()

    # def __del__(self):
    #     """
    #     del的调用时机不确定，依赖于Python的垃圾回收机制。
    #     :return:
    #     """
    #     self.close()


    def _get_instrument_address(self, address: str, config_path: str) -> str:
        """获取仪器地址

        优先使用传入的地址，否则从配置文件读取
        支持.yaml和.json格式的配置文件
        """
        if address:
            return address

        # 处理配置文件路径
        if not os.path.isabs(config_path):
            # 获取继承类的文件所在目录
            derived_class_file = inspect.getmodule(self.__class__).__file__
            derived_class_dir = os.path.dirname(
                os.path.abspath(derived_class_file))
            config_path = os.path.join(derived_class_dir, config_path)

        # 如果没有指定扩展名，先尝试.yaml，再尝试.json
        if not os.path.splitext(config_path)[1]:
            yaml_path = config_path + '.yaml'
            json_path = config_path + '.json'
            
            if os.path.exists(yaml_path):
                config_path = yaml_path
            elif os.path.exists(json_path):
                config_path = json_path
                
        config = load_config(config_path)
        address = config.get("instruments", {}).get(self.model)
        if not address:
            raise ValueError(f"未找到{self.model}的仪器地址配置")

        return address

    @handle_instrument_error
    def _connect_instrument(self) -> None:
        """建立仪器连接,返回仪器标识"""
        max_retries = 3  # 最大重试次数

        retry_count = 0
        
        while retry_count < max_retries:
            try:
                print(f"型号:{self.model}, 地址:{self.address}")
                self.rm = pyvisa.ResourceManager()
                self.instrument = self.rm.open_resource(self.address)

                self.instrument.write('*CLS')
                logging.info('清除仪器寄存器信息')


                # 验证连接
                idn = self.instrument.query("*IDN?")
                logging.info(f'仪器标识:{idn}, 成功连接{self.model}')
                return idn
            except(pyvisa.VisaIOError, pyvisa.VisaTypeError, TimeoutError):
                retry_count += 1
                if retry_count < max_retries:
                    logging.info(f"等待1秒后重试...")
                    time.sleep(1)
                    # 确保之前的连接被关闭
                    if hasattr(self, 'instrument'):
                        self.instrument.close()
                    if hasattr(self, 'rm'):
                        self.rm.close()
                else:
                    logging.error(f"连接失败次数超过最大重试次数({max_retries})")
                    raise

        return None


    @handle_instrument_error
    def write(self, command: str,
              check_complete: bool = False) -> None:
        """写入命令

        Args:
            command: SCPI命令
            check_complete: 是否等待命令完成

        example:
            self.write(f'SENSE1:FREQ:CENT 1GHz')
        """
        self.instrument.timeout = 100000
        self.instrument.write(command)
        if check_complete:
            self.wait_opc()
        logging.debug(f"{self.model} 写入: {command}")

    @handle_instrument_error
    def query(self, command: str,
              check_complete: bool = False) -> str:
        """查询命令

        Args:
            command: SCPI命令
            check_complete: 是否等待命令完成

        Returns:
            查询结果

        example:
            result = self.query("*IDN?")
            print(result)
        """
        self.instrument.timeout = 20000
        try:
            result = self.instrument.query(command)
            if check_complete:
                self.wait_opc()
            logging.debug(f"{self.model} 查询: {command}, 结果: {result}")
            return result
        except VisaIOError as e:
            logging.error(f"{self.model} 查询失败: {e}")
            raise
    
    @handle_instrument_error
    def set_opc_timeout(self, timeout: float=0.1, poll_interval: float = None):
        """临时设置OPC超时时间
        
        Args:
            timeout: 新的超时时间(秒)
            poll_interval: 新的轮询间隔(秒)，可选
        """
        self.opc_timeout = timeout
        if poll_interval is not None:
            self.opc_poll_interval = poll_interval
        logging.info(f"{self.model} 设置超时时间: {timeout}秒, 轮询间隔: {poll_interval or self.opc_poll_interval}秒")

    @handle_instrument_error
    def wait_opc(self, timeout=None, opc_poll_interval=None):
        """等待OPC完成
        Args:
            timeout: 超时时间(秒)，默认使用self.timeout
            opc_poll_interval: 轮询间隔(秒)，默认使用self.poll_interval
        """
        self.opc_timeout = timeout or self.opc_timeout
        try:
            start_time = time.time()
            logging.info(f"开始等待opc?")

            while True:
                try:
                    opc = self.instrument.query("*OPC?")
                    if opc.strip() == '1':
                        logging.info("opc运行完成")
                        break
                except VisaIOError as e:
                    logging.warning(f"OPC查询失败: {str(e)}")
                    # break  # 通信异常时终止

                elapsed = time.time() - start_time
                if elapsed >= self.opc_timeout:
                    logging.warning(f"运行超时（已耗时：{elapsed:.1f}秒）")
                    break
                time.sleep(opc_poll_interval or self.opc_poll_interval)
            end_time = time.time()
            total_time = end_time - start_time
            logging.info(f"总运行时间: {total_time:.2f}秒")
            return total_time  # 返回时间供外部使用
        except Exception as e:
            logging.error(f"发生错误: {str(e)}")
            return str(e)

    @handle_instrument_error
    def close(self) -> None:
        """关闭仪器连接"""
        self.instrument.close()
        logging.info(f"{self.model} 连接已关闭")




import os
import json
import yaml
import logging


def load_config(config_path="config.yaml"):
    """加载配置文件，支持YAML和JSON格式
    
    Args:
        config_path: 配置文件路径(.yaml或.json)
        
    Returns:
        配置字典
    """
    if not os.path.exists(config_path):
        logging.error(f"配置文件未找到: {config_path}")
        raise FileNotFoundError(f"配置文件未找到: {config_path}")
        
    file_ext = os.path.splitext(config_path)[1].lower()
    
    with open(config_path, "r", encoding="utf-8") as file:
        try:
            if file_ext in ['.yaml', '.yml']:
                return yaml.safe_load(file)
            elif file_ext == '.json':
                return json.load(file)
            else:
                error_msg = f"不支持的配置文件格式: {file_ext}"
                logging.error(error_msg)
                raise ValueError(error_msg)
        except (yaml.YAMLError, json.JSONDecodeError) as e:
            error_msg = f"配置文件格式错误: {e}"
            logging.error(error_msg)
            raise ValueError(error_msg)

def get_config_path(config_path):
    """获取当前工作目录"""
    # 获取当前工作目录
    current_dir = os.getcwd()
    # 拼接配置文件的路径
    config_path = os.path.join(current_dir, config_path)
    return config_path


if __name__ == '__main__':
    config = get_config_path("config.yaml")


def _get_version() -> str:

    from importlib.resources import files
    from pathlib import Path

    import versioningit

    module_path = files("pyinsts")
    if isinstance(module_path, Path):
        return versioningit.get_version(project_dir=Path(module_path).parent.parent)
    else:
        return "0.0"


__version__ = "0.0.3"
